mod agcd;
mod file;
mod lll;
mod matrix;
mod utils;

use crate::agcd::agcd;
use crate::file::parse_file;
use clap::{Parser, Subcommand};
use std::path::{Path, PathBuf};

#[derive(Parser)]
#[command(author, version, about, long_about = None)]
#[command(propagate_version = true)]
struct Cli {
    #[command(subcommand)]
    command: Commands,
}

#[derive(Subcommand)]
enum Commands {
    /// approximate GCD
    Agcd {
        /// (default './input.txt')
        path: Option<PathBuf>,
    },
}

fn main() -> std::io::Result<()> {
    let cli = Cli::parse();

    match &cli.command {
        Commands::Agcd { path } => {
            let path = path.as_deref().unwrap_or(Path::new("./input.txt"));
            let input = parse_file(path)?;

            let result = agcd(input.numbers, input.noise_bits);
            println!(
                "Approximate GCD with noise_bits={}: {}",
                input.noise_bits, result
            );
        }
    }
    Ok(())
}
